{ config, pkgs, ... }:

let
  shell = "${pkgs.dash}/bin/dash";

  riverctl = "${pkgs.river}/bin/riverctl";

  foot = "${pkgs.foot}/bin/footclient";
  bemenu-run = "${pkgs.bemenu}/bin/bemenu-run";
  tessen = "${pkgs.tessen}/bin/tessen";
  swaylock = "${pkgs.swaylock}/bin/swaylock";
  eject = "${pkgs.util-linux}/bin/eject";
  pamixer = "${pkgs.pamixer}/bin/pamixer";
  playerctl = "${pkgs.playerctl}/bin/playerctl";
  light = "${pkgs.light}/bin/light";

  seq = "${pkgs.coreutils}/bin/seq";

  dbus-update-activation-environment =
    "${pkgs.dbus}/bin/dbus-update-activation-environment";

  configure-gtk = let
    schema = pkgs.gsettings-desktop-schemas;
    datadir = "${schema}/share/gsettings-schemas/${schema.name}";
    gsettings = "${pkgs.glib}/bin/gsettings";
  in pkgs.writeScriptBin "configure-gtk" ''
    #!${shell}

    export XDG_DATA_DIRS=${datadir}:$XDG_DATA_DIRS

    ${gsettings} set org.gnome.desktop.interface gtk-theme 'gruvbox-dark'
    ${gsettings} set org.gnome.desktop.interface icon-theme 'Papirus-Dark'
    ${gsettings} set org.gnome.desktop.wm.preferences button-layout ""
  '';

in {
  xdg.dataFile.river_init = {
    executable = true;
    target = "${config.xdg.configHome}/river/init";
    text = ''
      #!${shell}

      ${riverctl} map normal Super Return spawn ${foot}
      ${riverctl} map normal Super R spawn ${bemenu-run}
      ${riverctl} map normal Super P spawn ${tessen}
      ${riverctl} map normal Super Escape spawn ${swaylock}

      # Super+Shift+C to close the focused view
      ${riverctl} map normal Super+Shift C close

      # Super+Shift+Esc to exit river
      # ${riverctl} map normal Super+Shift Escape exit


      # Super+J and Super+K to focus the next/previous view in the layout stack
      ${riverctl} map normal Super J focus-view next
      ${riverctl} map normal Super K focus-view previous

      # Super+Shift+J and Super+Shift+K to swap the focused view with the next/previous
      # view in the layout stack
      ${riverctl} map normal Super+Shift J swap next
      ${riverctl} map normal Super+Shift K swap previous

      # Super+E and Super+W to focus the next/previous output
      ${riverctl} map normal Super E focus-output next
      ${riverctl} map normal Super W focus-output previous

      # Super+Shift+{W,E} to send the focused view to the next/previous output
      ${riverctl} map normal Super+Shift W send-to-output next
      ${riverctl} map normal Super+Shift W send-to-output previous

      # Super+Shift+Return to bump the focused view to the top of the layout stack
      ${riverctl} map normal Super+Shift Return zoom


      ${riverctl} map normal Super+Alt H move left 100
      ${riverctl} map normal Super+Alt J move down 100
      ${riverctl} map normal Super+Alt K move up 100
      ${riverctl} map normal Super+Alt L move right 100

      # Super+Alt+Control+{H,J,K,L} to snap views to screen edges
      ${riverctl} map normal Super+Alt+Control H snap left
      ${riverctl} map normal Super+Alt+Control J snap down
      ${riverctl} map normal Super+Alt+Control K snap up
      ${riverctl} map normal Super+Alt+Control L snap right

      # Super+Alt+Shift+{H,J,K,L} to resize views
      ${riverctl} map normal Super+Alt+Shift H resize horizontal -100
      ${riverctl} map normal Super+Alt+Shift J resize vertical 100
      ${riverctl} map normal Super+Alt+Shift K resize vertical -100
      ${riverctl} map normal Super+Alt+Shift L resize horizontal 100

      # Super + Left Mouse Button to move views
      ${riverctl} map-pointer normal Super BTN_LEFT move-view

      # Super + Right Mouse Button to resize views
      ${riverctl} map-pointer normal Super BTN_RIGHT resize-view

      # Super + Middle Mouse Button to toggle float
      ${riverctl} map-pointer normal Super BTN_MIDDLE toggle-float

      for i in $(${seq} 1 9)
      do
          tags=$((1 << (i - 1)))

          # Super+[1-9] to focus tag [0-8]
          ${riverctl} map normal Super "$i" set-focused-tags "$tags"

          # Super+Shift+[1-9] to tag focused view with tag [0-8]
          ${riverctl} map normal Super+Shift "$i" set-view-tags "$tags"

          # Super+Ctrl+[1-9] to toggle focus of tag [0-8]
          ${riverctl} map normal Super+Control "$i" toggle-focused-tags "$tags"

          # Super+Shift+Ctrl+[1-9] to toggle tag [0-8] of focused view
          ${riverctl} map normal Super+Shift+Control "$i" toggle-view-tags "$tags"
      done

      # Super+0 to focus all tags
      # Super+Shift+0 to tag focused view with all tags
      all_tags=$(((1 << 32) - 1))
      ${riverctl} map normal Super 0 set-focused-tags "$all_tags"
      ${riverctl} map normal Super+Shift 0 set-view-tags "$all_tags"

      # Super+T to toggle float
      ${riverctl} map normal Super T toggle-float

      # Super+M to toggle fullscreen
      ${riverctl} map normal Super M toggle-fullscreen

      # Declare a passthrough mode. This mode has only a single mapping to return to
      # normal mode. This makes it useful for testing a nested wayland compositor
      ${riverctl} declare-mode passthrough

      # Super+F11 to enter passthrough mode
      ${riverctl} map normal Super F11 enter-mode passthrough

      # Super+F11 to return to normal mode
      ${riverctl} map passthrough Super F11 enter-mode normal

      # Various media key mapping examples for both normal and locked mode which do
      # not have a modifier
      for mode in normal locked
      do
          # Eject the optical drive (well if you still have one that is)
          ${riverctl} map $mode None XF86Eject spawn '${eject} -T'

          # Control pulse audio volume with pamixer (https://github.com/cdemoulins/pamixer)
          ${riverctl} map $mode None XF86AudioRaiseVolume  spawn '${pamixer} -i 5'
          ${riverctl} map $mode None XF86AudioLowerVolume  spawn '${pamixer} -d 5'
          ${riverctl} map $mode None XF86AudioMute         spawn '${pamixer} --toggle-mute'

          # Control MPRIS aware media players with playerctl (https://github.com/altdesktop/playerctl)
          ${riverctl} map $mode None XF86AudioMedia spawn '${playerctl} play-pause'
          ${riverctl} map $mode None XF86AudioPlay  spawn '${playerctl} play-pause'
          ${riverctl} map $mode None XF86AudioPrev  spawn '${playerctl} previous'
          ${riverctl} map $mode None XF86AudioNext  spawn '${playerctl} next'

          # Control screen backlight brightness with light (https://github.com/haikarainen/light)
          ${riverctl} map $mode None XF86MonBrightnessUp   spawn '${light} -A 5'
          ${riverctl} map $mode None XF86MonBrightnessDown spawn '${light} -U 5'
      done

      # Set background and border color
      ${riverctl} background-color 0x282828
      ${riverctl} border-color-focused 0x458588
      ${riverctl} border-color-unfocused 0x504945
      ${riverctl} border-width 3

      ${riverctl} focus-follows-cursor normal

      # Set keyboard repeat rate
      ${riverctl} set-repeat 50 300

      # Make certain views start floating
      ${riverctl} float-filter-add app-id float
      ${riverctl} float-filter-add title "popup title with spaces"

      # Set app-ids and titles of views which should use client side decorations
      ${riverctl} csd-filter-add app-id "gedit"

      ${configure-gtk}/bin/configure-gtk

      trackball_names="pointer-21298-5120-Clearly_Superior_Technologies._CST_Laser_Trackball 21298:5120:Clearly_Superior_Technologies._CST_Laser_Trackball"
      for name in $trackball_names; do
          ${riverctl} input "$name" left-handed enabled
          ${riverctl} input "$name" pointer-accel -0.8
      done

      ${config.xdg.configHome}/river/init-kile

      export XDG_CURRENT_DESKTOP=river
      ${dbus-update-activation-environment} --systemd DISPLAY WAYLAND_DISPLAY XDG_CURRENT_DESKTOP
      systemctl --user import-environment DISPLAY WAYLAND_DISPLAY XDG_CURRENT_DESKTOP
      systemctl --user start river-session.target
    '';
  };
}
